@@ -9,7 +9,9 @@ from argh import arg, ArghParser
 from .gantry import Gantry, GantryError, DOCKER_DEFAULT_URL
 
 _log_level_default = logging.WARN
-_log_level = getattr(logging, os.environ.get('GANTRY_LOGLEVEL', '').upper(), _log_level_default)
+_log_level = getattr(logging,
+                     os.environ.get('GANTRY_LOGLEVEL', '').upper(),
+                     _log_level_default)
 logging.basicConfig(format='%(levelname)s: %(message)s', level=_log_level)
 
 
